from django.conf import settings
from django.test.testcases import TestCase
from django.test.utils import override_settings
from django.urls import reverse

from dbfiles.models import DBFile


class DBFileViewTests(TestCase):
    def setUp(self):
        super(DBFileViewTests, self).setUp()
        self.name = "test/hello_world.txt"
        self.db_file = DBFile.objects.create(content=b"Hello World!", name=self.name)
        self.url = reverse("db_file", args=[self.name])

    def test_get(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["Content-Type"], "text/plain")
        self.assertEqual(response["Content-Length"], "12")
        self.assertFalse("Content-Encoding" in response)
        self.assertTrue("Last-Modified" in response)

    @override_settings(TIME_ZONE="America/Toronto")
    def test_get_not_utc(self):
        self.assertEqual(settings.TIME_ZONE, "America/Toronto")
        self.test_get()

    @override_settings(USE_TZ=False)
    def test_get_tz_disabled(self):
        self.assertFalse(settings.USE_TZ)
        self.test_get()

    def test_get_not_modified(self):
        response = self.client.get(self.url)
        self.assertEqual(response.status_code, 200)
        self.assertTrue("Last-Modified" in response)

        response = self.client.get(
            path="/media/{}".format(self.name),
            HTTP_IF_MODIFIED_SINCE=response["Last-Modified"],
        )
        self.assertEqual(response.status_code, 304)

    def test_get_modified(self):
        self.db_file.content = b"Updated Content!"
        self.db_file.save()

        response = self.client.get(
            path=self.url, HTTP_IF_MODIFIED_SINCE="invalid-last-modified"
        )
        self.assertEqual(response.status_code, 200)

    def test_get_not_modified_bad_string(self):
        response = self.client.get(
            path=self.url, HTTP_IF_MODIFIED_SINCE="invalid-last-modified"
        )
        self.assertEqual(response.status_code, 200)



from datetime import timedelta

from django.conf import settings
from django.core.files.base import ContentFile
from django.test.testcases import TestCase
from django.test.utils import override_settings
from django.utils import timezone

from dbfiles.models import DBFile
from dbfiles.storage import DBStorage


class DBStorageTests(TestCase):
    storage_class = DBStorage

    def setUp(self):
        self.storage = self.storage_class("/test_media_url/")

    def test_default_base_url(self):
        storage = self.storage_class()
        self.assertEqual(storage.base_url, settings.MEDIA_URL)

    def test_default_base_url_append_slash(self):
        storage = self.storage_class("/test")
        self.assertEqual(storage.base_url, "/test/")

    def test_file_access_options(self):
        self.assertFalse(self.storage.exists("storage_test"))

        f = ContentFile(b"storage contents")
        self.storage.save("storage_test", f)
        self.assertEqual(self.storage.size("storage_test"), 16)
        self.assertNumQueries(1, self.storage.size, "storage_test")

        f = self.storage.open("storage_test")
        self.assertEqual(f.read(), b"storage contents")

        self.storage.delete("storage_test")
        self.assertFalse(self.storage.exists("storage_test"))

    def test_file_created_time(self):
        name = "test.file"
        DBFile.objects.create(content=b"custom content", name=name)
        ctime = self.storage.created_time(name)

        self.assertEqual(DBFile.objects.get(name=name).created_on, ctime)
        self.assertLess(
            timezone.now() - self.storage.created_time(name), timedelta(seconds=1)
        )
        self.assertNumQueries(1, self.storage.created_time, name)

    def test_file_modified_time(self):
        name = "test.file"
        DBFile.objects.create(content=b"custom content", name=name)
        mtime = self.storage.modified_time(name)

        self.assertEqual(DBFile.objects.get(name=name).updated_on, mtime)
        self.assertLess(
            timezone.now() - self.storage.modified_time(name), timedelta(seconds=1)
        )
        self.assertNumQueries(1, self.storage.modified_time, name)

    @override_settings(USE_TZ=False)
    def test_file_created_time_tz_disabled(self):
        self.test_file_created_time()

    @override_settings(USE_TZ=False)
    def test_file_modified_time_tz_disabled(self):
        self.test_file_modified_time()

    def test_file_save_without_name(self):
        self.assertFalse(self.storage.exists("test.file"))

        f = ContentFile(b"custom contents")
        f.name = "test.file"

        storage_f_name = self.storage.save(None, f)
        self.assertEqual(storage_f_name, f.name)
        self.assertTrue(DBFile.objects.filter(name=f.name).exists())

    def test_file_url(self):
        self.assertEqual(
            self.storage.url("test.file"),
            "{}{}".format(self.storage.base_url, "test.file"),
        )

        self.assertEqual(
            self.storage.url(r"""~!*()'@#$%^&*abc`+ =.file"""),
            """/test_media_url/~!*()'%40%23%24%25%5E%26*abc%60%2B%20%3D.file""",
        )

    def test_get_valid_url_max_length(self):
        name = "test" * 100
        valid_name = self.storage.get_valid_name(name)
        self.assertEqual(valid_name, name[:255])

    def test_path_not_implemented(self):
        self.assertRaises(NotImplementedError, self.storage.path, "")

    def test_listdir_not_implemented(self):
        self.assertRaises(NotImplementedError, self.storage.listdir, "")

    def test_accessed_time_not_implemented(self):
        self.assertRaises(NotImplementedError, self.storage.accessed_time, "")



from django.test.testcases import TestCase

import dbfiles
from dbfiles.apps import DBFilesConfig


class DBFilesConfigTests(TestCase):
    def test_app_config(self):
        config = DBFilesConfig("dbfiles", dbfiles)
        self.assertEqual(config.verbose_name, "DB Files")



from django.contrib.auth.models import User
from django.test.testcases import TestCase
from django.urls import reverse

from dbfiles.models import DBFile


class DBFileAdminTests(TestCase):
    def setUp(self):
        super(DBFileAdminTests, self).setUp()
        self.name = "test/hello_world.txt"
        self.db_file = DBFile.objects.create(content=b"Hello World!", name=self.name)
        self.url_parts = {
            "app_label": self.db_file._meta.app_label,
            "model_name": self.db_file._meta.model_name,
        }

        User.objects.create_superuser("superuser", "", "password")
        logged_in = self.client.login(username="superuser", password="password")
        self.assertTrue(logged_in)

    def test_admin_list_view(self):
        url = reverse(
            "admin:{app_label}_{model_name}_changelist".format(**self.url_parts)
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)

    def test_admin_details_view(self):
        url = reverse(
            "admin:{app_label}_{model_name}_change".format(**self.url_parts),
            args=[self.db_file.pk],
        )
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)



from __future__ import unicode_literals

SECRET_KEY = "fake-key"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "dbfiles",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

DATABASES = {
    "default": {"ENGINE": "django.db.backends.sqlite3", "NAME": ":memory:"},
}

ROOT_URLCONF = "tests.urls"

MEDIA_URL = "/media/"

DEFAULT_FILE_STORAGE = "dbfiles.storage.DBStorage"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

TIME_ZONE = "UTC"
USE_TZ = True



from django.core.files.uploadedfile import SimpleUploadedFile
from django.test.testcases import TestCase

from dbfiles.forms import DBFileForm, DBFileWidget
from dbfiles.models import DBFile


class DBFileFormTests(TestCase):
    def test_save(self):
        form = DBFileForm(
            data={"name": "hello-world.txt"},
            files={"file": SimpleUploadedFile("test.file", b"Hello World!")},
        )
        self.assertTrue(form.is_valid(), form.errors)

        db_file = form.save()
        self.assertTrue(db_file.pk)
        self.assertEqual(db_file.size, 12)
        self.assertEqual(db_file.content, b"Hello World!")
        self.assertEqual(db_file.name, "hello-world.txt")

    def test_save_with_instance(self):
        content = b"Hello World!"
        name = "my-files/hello-world.txt"

        db_file = DBFile.objects.create(content=content, name=name)
        form = DBFileForm(instance=db_file, data={"name": "my-files/renamed.txt"})
        self.assertTrue(form.is_valid())

        new_db_file = form.save()
        self.assertEqual(db_file.pk, new_db_file.pk)
        self.assertEqual(db_file.content, new_db_file.content)
        self.assertEqual("my-files/renamed.txt", new_db_file.name)

    def test_db_file_widget(self):
        widget = DBFileWidget()
        widget.instance = None
        html = widget.render("file", "")
        self.assertRegex(html, r'<input\s+type="file"\s+name="file"\s*/?>')

    def test_db_file_widget_with_instance(self):
        content = b"Hello World!"
        name = "my-files/hello-world.txt"
        db_file = DBFile.objects.create(content=content, name=name)

        widget = DBFileWidget()
        widget.instance = db_file
        html = widget.render("file", "")
        self.assertTrue("<table>" in html)



from django.contrib import admin
from django.urls import path

from dbfiles.urls import dbfiles_url

urlpatterns = [
    # admin urls
    path("admin/", admin.site.urls),
    # dbfiles
    dbfiles_url(),
]



from django.test.testcases import TestCase

from dbfiles.models import DBFile


class DBFileTests(TestCase):
    def test_db_file_save(self):
        content = b"Hello World!"
        name = "my-files/hello-world.txt"

        db_file = DBFile.objects.create(content=content, name=name)

        self.assertEqual(db_file.content, content)
        self.assertEqual(db_file.name, name)
        self.assertEqual(db_file.size, len(content))
        self.assertTrue(db_file.created_on)
        self.assertTrue(db_file.updated_on)
        self.assertEqual(str(db_file), name)
